#!/bin/sh
set -ex

yum install -y centos-release-scl
yum -y update

# Basic system utilities
yum -y install make git

# Java stuff
yum -y install apache-maven java-11-openjdk-devel

update-alternatives --set java $(realpath /usr/lib/jvm/java-11-openjdk/bin/java)
update-alternatives --set javac $(realpath /usr/lib/jvm/java-11-openjdk/bin/javac)

# Print out java version
java -version
javac -version
mvn --version

# Install developer toolset

yum -y install                     \
  devtoolset-7-toolchain           \
  devtoolset-7-libasan-devel       \
  devtoolset-7-libubsan-devel      \
  devtoolset-7-valgrind            \
  devtoolset-7-perftools           \
  devtoolset-7-gdb-gdbserver       \
  llvm-toolset-7                   \
  llvm-toolset-7-clang-tools-extra

# Install patch (needed to patch nginx)
yum -y install patch

# Install CMAKE
CMAKE_DOWNLOAD_URL=${CMAKE_DOWNLOAD_URL:-https://cmake.org/files/v3.17/cmake-3.17.1-Linux-x86_64.sh}

wget --no-verbose ${CMAKE_DOWNLOAD_URL} -O /tmp/install-cmake.sh
mkdir /opt/cmake && sh /tmp/install-cmake.sh --skip-license --prefix=/opt/cmake  --exclude-subdir

cat << EOF > /etc/profile.d/90enable-custom-cmake.sh
#!/bin/bash
export PATH="/opt/cmake/bin:${PATH}"
EOF

source /etc/profile.d/90enable-custom-cmake.sh
cmake --version
rm /tmp/install-cmake.sh


# Setup build user .m2 folder
mkdir ${BUILD_USER_HOME}/.m2
cp /docker/settings.xml ${BUILD_USER_HOME}/.m2
mkdir /m2-repository

chown -R ${BUILD_USER}:${BUILD_USER} ${BUILD_USER_HOME} /m2-repository

# Install boost-devel & doxygen
yum -y install doxygen

# Cleanup to save some space
yum clean all

# Install GRPC build-time dependencies
yum -y install gflags-devel zlib-devel openssl-devel

# Install test coverage evaluation tools
yum -y install lcov

# Install libattr for extended attributes support
yum -y install libattr-devel

# Install libacl for posix acl support
yum -y install libacl-devel

rm -rf /var/cache/yum

# build dependencies with the devtoolset compilers
source /opt/rh/devtoolset-7/enable
source /opt/rh/llvm-toolset-7/enable

# can't use the following because it exits with 1, failing the script
# source scl_source enable devtoolset-7 llvm-toolset-7

gcc --version
clang --version
clang-tidy --version
clang-format --version

# Install boost
mkdir /tmp/boost-build
cd /tmp/boost-build
wget --no-verbose https://boostorg.jfrog.io/artifactory/main/release/1.72.0/source/boost_1_72_0.tar.gz -O boost.tar.gz
mkdir boost-src && tar zxf boost.tar.gz -C boost-src --strip-components=1
cd boost-src
./bootstrap.sh --with-libraries=filesystem,log,program_options,system,thread
mkdir /tmp/boost-build-static /tmp/boost-build-shared
./b2 --build-dir=/tmp/boost-build-static toolset=gcc version=release link=static install
./b2 --build-dir=/tmp/boost-build-shared toolset=gcc version=release link=shared install

# Install gtest and gmock
mkdir /tmp/googletest-build
cd /tmp/googletest-build
git clone https://github.com/google/googletest.git
cd googletest
mkdir build && cd build
cmake ..
make -j$(nproc)
make install

# gRPC
# build with cmake
# * to install correctly, the c-ares, protobuf, zlib and ssl deps have
#   to be of "package" type
# * c-ares and protobuf are built and installed from the submodules,
#   zlib and ssl are pre-installed
# * see https://github.com/grpc/grpc/blob/master/test/distrib/cpp/run_distrib_test_cmake.sh

GRPC_BRANCH=${GRPC_BRANCH:-v1.33.2}

mkdir /tmp/grpc-build
cd /tmp/grpc-build
git clone -b ${GRPC_BRANCH} https://github.com/grpc/grpc.git
cd grpc
git submodule update --init

# c-ares

mkdir -p third_party/cares/cares/cmake/build
cd third_party/cares/cares/cmake/build
cmake -DCMAKE_BUILD_TYPE=Release ../..
make -j$(nproc) install
cd -
rm -rf third_party/cares/cares  # wipe out to prevent influencing the grpc build

# protobuf

mkdir -p third_party/protobuf/cmake/build
cd third_party/protobuf/cmake/build
cmake -Dprotobuf_BUILD_TESTS=OFF -DCMAKE_BUILD_TYPE=Release ..
make -j$(nproc) install
cd -
rm -rf third_party/protobuf  # wipe out to prevent influencing the grpc build

echo /usr/local/lib > /etc/ld.so.conf.d/usrlocal.conf
ldconfig

# gRPC (finally)

mkdir -p build
cd build
cmake -DgRPC_BUILD_TESTS=OFF -DgRPC_PROTOBUF_PROVIDER=package \
      -DgRPC_ZLIB_PROVIDER=package -DgRPC_CARES_PROVIDER=package \
      -DgRPC_SSL_PROVIDER=package -DgRPC_GFLAGS_PROVIDER=package \
      -DCMAKE_BUILD_TYPE=Release \
      ..
make -j$(nproc) install
cd -

ldconfig

# yaml-cpp (only shared libs for the moment)
mkdir /tmp/yaml-cpp-build
cd /tmp/yaml-cpp-build
wget --no-verbose https://github.com/jbeder/yaml-cpp/archive/yaml-cpp-0.6.2.tar.gz -O yaml.tar.gz
mkdir yaml-src && tar zxf yaml.tar.gz -C yaml-src --strip-components=1
cd yaml-src
mkdir build && cd build
cmake -DBUILD_SHARED_LIBS=1 -DYAML_CPP_BUILD_TESTS=0 -DYAML_CPP_BUILD_TOOLS=0 ..
make -j$(nproc)
make install

# Cleanup to save some space
cd /
yum clean all
rm -rf /tmp/boost-build
rm -rf /tmp/boost-build-static
rm -rf /tmp/boost-build-shared
rm -rf /tmp/grpc-build
rm -rf /tmp/yaml-cpp-build
rm -rf /tmp/googletest-build

# Cleanup docker image assets
# Currently commented out as it fails for unknown reason on the gitlab ci
# runner
# rm -rfv /docker
